@@ -7,31 +7,21 @@ import re
 import warnings
 from fnmatch import fnmatch
 
-from nbformat import sign
+from jupyter_core.utils import ensure_async
+from jupyter_events import EventLogger
+from nbformat import ValidationError, sign
 from nbformat import validate as validate_nb
-from nbformat import ValidationError
 from nbformat.v4 import new_notebook
-from tornado.web import HTTPError
-from tornado.web import RequestHandler
-from traitlets import Any
-from traitlets import Bool
-from traitlets import default
-from traitlets import Dict
-from traitlets import Instance
-from traitlets import List
-from traitlets import TraitError
-from traitlets import Type
-from traitlets import Unicode
-from traitlets import validate
+from tornado.web import HTTPError, RequestHandler
+from traitlets import Any, Bool, Dict, Instance, List, TraitError, Type, Unicode, default, validate
 from traitlets.config.configurable import LoggingConfigurable
 
-from ...files.handlers import FilesHandler
-from .checkpoints import AsyncCheckpoints
-from .checkpoints import Checkpoints
+from jupyter_server import DEFAULT_EVENTS_SCHEMA_PATH, JUPYTER_SERVER_EVENTS_URI
 from jupyter_server.transutils import _i18n
-from jupyter_server.utils import ensure_async
 from jupyter_server.utils import import_item
 
+from ...files.handlers import FilesHandler
+from .checkpoints import AsyncCheckpoints, Checkpoints
 
 copy_pat = re.compile(r"\-Copy\d*\.")
 
@@ -55,6 +45,24 @@ class ContentsManager(LoggingConfigurable):
 
     """
 
+    event_schema_id = JUPYTER_SERVER_EVENTS_URI + "/contents_service/v1"
+    event_logger = Instance(EventLogger).tag(config=True)
+
+    @default("event_logger")
+    def _default_event_logger(self):
+        if self.parent and hasattr(self.parent, "event_logger"):
+            return self.parent.event_logger
+        else:
+            # If parent does not have an event logger, create one.
+            logger = EventLogger()
+            schema_path = DEFAULT_EVENTS_SCHEMA_PATH / "contents_service" / "v1.yaml"
+            logger.register_event_schema(schema_path)
+            return logger
+
+    def emit(self, data):
+        """Emit event using the core event schema from Jupyter Server's Contents Manager."""
+        self.event_logger.emit(schema_id=self.event_schema_id, data=data)
+
     root_dir = Unicode("/", config=True)
 
     allow_hidden = Bool(False, config=True, help="Allow access to hidden files")
@@ -127,7 +135,7 @@ class ContentsManager(LoggingConfigurable):
             value = import_item(self.pre_save_hook)
         if not callable(value):
             raise TraitError("pre_save_hook must be callable")
-        if self.pre_save_hook is not None:
+        if callable(self.pre_save_hook):
             warnings.warn(
                 f"Overriding existing pre_save_hook ({self.pre_save_hook.__name__}) with a new one ({value.__name__}).",
                 stacklevel=2,
@@ -162,7 +170,7 @@ class ContentsManager(LoggingConfigurable):
             value = import_item(value)
         if not callable(value):
             raise TraitError("post_save_hook must be callable")
-        if self.post_save_hook is not None:
+        if callable(self.post_save_hook):
             warnings.warn(
                 f"Overriding existing post_save_hook ({self.post_save_hook.__name__}) with a new one ({value.__name__}).",
                 stacklevel=2,
@@ -270,10 +278,10 @@ class ContentsManager(LoggingConfigurable):
 
     @default("checkpoints_kwargs")
     def _default_checkpoints_kwargs(self):
-        return dict(
-            parent=self,
-            log=self.log,
-        )
+        return {
+            "parent": self,
+            "log": self.log,
+        }
 
     files_handler_class = Type(
         FilesHandler,
@@ -418,11 +426,13 @@ class ContentsManager(LoggingConfigurable):
             raise HTTPError(400, "Can't delete root")
         self.delete_file(path)
         self.checkpoints.delete_all_checkpoints(path)
+        self.emit(data={"action": "delete", "path": path})
 
     def rename(self, old_path, new_path):
         """Rename a file and any checkpoints associated with that file."""
         self.rename_file(old_path, new_path)
         self.checkpoints.rename_all_checkpoints(old_path, new_path)
+        self.emit(data={"action": "rename", "path": new_path, "source_path": old_path})
 
     def update(self, model, path):
         """Update the file's path
@@ -479,13 +489,13 @@ class ContentsManager(LoggingConfigurable):
 
         for i in itertools.count():
             if i:
-                insert_i = "{}{}".format(insert, i)
+                insert_i = f"{insert}{i}"
             else:
                 insert_i = ""
             name = "{basename}{insert}{suffix}".format(
                 basename=basename, insert=insert_i, suffix=suffix
             )
-            if not self.exists("{}/{}".format(path, name)):
+            if not self.exists(f"{path}/{name}"):
                 break
         return name
 
@@ -545,7 +555,7 @@ class ContentsManager(LoggingConfigurable):
             raise HTTPError(400, "Unexpected model type: %r" % model["type"])
 
         name = self.increment_filename(untitled + ext, path, insert=insert)
-        path = "{0}/{1}".format(path, name)
+        path = f"{path}/{name}"
         return self.new(model, path)
 
     def new(self, model=None, path=""):
@@ -586,6 +596,7 @@ class ContentsManager(LoggingConfigurable):
         from_path must be a full path to a file.
         """
         path = from_path.strip("/")
+
         if to_path is not None:
             to_path = to_path.strip("/")
 
@@ -601,14 +612,23 @@ class ContentsManager(LoggingConfigurable):
         if model["type"] == "directory":
             raise HTTPError(400, "Can't copy directories")
 
-        if to_path is None:
+        is_destination_specified = to_path is not None
+        if not is_destination_specified:
             to_path = from_dir
         if self.dir_exists(to_path):
             name = copy_pat.sub(".", from_name)
             to_name = self.increment_filename(name, to_path, insert="-Copy")
-            to_path = "{0}/{1}".format(to_path, to_name)
+            to_path = f"{to_path}/{to_name}"
+        elif is_destination_specified:
+            if "/" in to_path:
+                to_dir, to_name = to_path.rsplit("/", 1)
+                if not self.dir_exists(to_dir):
+                    raise HTTPError(404, "No such parent directory: %s to copy file in" % to_dir)
+        else:
+            raise HTTPError(404, "No such directory: %s" % to_path)
 
         model = self.save(model, to_path)
+        self.emit(data={"action": "copy", "path": to_path, "source_path": from_path})
         return model
 
     def log_info(self):
@@ -697,10 +717,10 @@ class AsyncContentsManager(ContentsManager):
 
     @default("checkpoints_kwargs")
     def _default_checkpoints_kwargs(self):
-        return dict(
-            parent=self,
-            log=self.log,
-        )
+        return {
+            "parent": self,
+            "log": self.log,
+        }
 
     # ContentsManager API part 1: methods that must be
     # implemented in subclasses.
@@ -812,11 +832,13 @@ class AsyncContentsManager(ContentsManager):
 
         await self.delete_file(path)
         await self.checkpoints.delete_all_checkpoints(path)
+        self.emit(data={"action": "delete", "path": path})
 
     async def rename(self, old_path, new_path):
         """Rename a file and any checkpoints associated with that file."""
         await self.rename_file(old_path, new_path)
         await self.checkpoints.rename_all_checkpoints(old_path, new_path)
+        self.emit(data={"action": "rename", "path": new_path, "source_path": old_path})
 
     async def update(self, model, path):
         """Update the file's path
@@ -858,13 +880,13 @@ class AsyncContentsManager(ContentsManager):
 
         for i in itertools.count():
             if i:
-                insert_i = "{}{}".format(insert, i)
+                insert_i = f"{insert}{i}"
             else:
                 insert_i = ""
             name = "{basename}{insert}{suffix}".format(
                 basename=basename, insert=insert_i, suffix=suffix
             )
-            file_exists = await ensure_async(self.exists("{}/{}".format(path, name)))
+            file_exists = await ensure_async(self.exists(f"{path}/{name}"))
             if not file_exists:
                 break
         return name
@@ -905,7 +927,7 @@ class AsyncContentsManager(ContentsManager):
             raise HTTPError(400, "Unexpected model type: %r" % model["type"])
 
         name = await self.increment_filename(untitled + ext, path, insert=insert)
-        path = "{0}/{1}".format(path, name)
+        path = f"{path}/{name}"
         return await self.new(model, path)
 
     async def new(self, model=None, path=""):
@@ -946,6 +968,7 @@ class AsyncContentsManager(ContentsManager):
         from_path must be a full path to a file.
         """
         path = from_path.strip("/")
+
         if to_path is not None:
             to_path = to_path.strip("/")
 
@@ -960,14 +983,24 @@ class AsyncContentsManager(ContentsManager):
         model.pop("name", None)
         if model["type"] == "directory":
             raise HTTPError(400, "Can't copy directories")
-        if to_path is None:
+
+        is_destination_specified = to_path is not None
+        if not is_destination_specified:
             to_path = from_dir
         if await ensure_async(self.dir_exists(to_path)):
             name = copy_pat.sub(".", from_name)
             to_name = await self.increment_filename(name, to_path, insert="-Copy")
-            to_path = "{0}/{1}".format(to_path, to_name)
+            to_path = f"{to_path}/{to_name}"
+        elif is_destination_specified:
+            if "/" in to_path:
+                to_dir, to_name = to_path.rsplit("/", 1)
+                if not await ensure_async(self.dir_exists(to_dir)):
+                    raise HTTPError(404, "No such parent directory: %s to copy file in" % to_dir)
+        else:
+            raise HTTPError(404, "No such directory: %s" % to_path)
 
         model = await self.save(model, to_path)
+        self.emit(data={"action": "copy", "path": to_path, "source_path": from_path})
         return model
 
     async def trust_notebook(self, path):
