@@ -7,16 +7,16 @@
 # Distributed under the terms of the Modified BSD License.
 import asyncio
 import os
+import warnings
 from collections import defaultdict
 from datetime import datetime, timedelta
 from functools import partial
 
-from jupyter_client.multikernelmanager import (
-    AsyncMultiKernelManager,
-    MultiKernelManager,
-)
+from jupyter_client.ioloop.manager import AsyncIOLoopKernelManager
+from jupyter_client.multikernelmanager import AsyncMultiKernelManager, MultiKernelManager
 from jupyter_client.session import Session
 from jupyter_core.paths import exists
+from jupyter_core.utils import ensure_async
 from tornado import web
 from tornado.concurrent import Future
 from tornado.ioloop import IOLoop, PeriodicCallback
@@ -36,7 +36,7 @@ from traitlets import (
 
 from jupyter_server._tz import isoformat, utcnow
 from jupyter_server.prometheus.metrics import KERNEL_CURRENTLY_RUNNING_TOTAL
-from jupyter_server.utils import ensure_async, to_os_path
+from jupyter_server.utils import import_item, to_os_path
 
 
 class MappingKernelManager(MultiKernelManager):
@@ -180,7 +180,7 @@ class MappingKernelManager(MultiKernelManager):
         self.log.warning("Kernel %s died, removing from map.", kernel_id)
         self.remove_kernel(kernel_id)
 
-    def cwd_for_path(self, path):
+    def cwd_for_path(self, path, **kwargs):
         """Turn API path into absolute OS path."""
         os_path = to_os_path(path, self.root_dir)
         # in the case of documents and kernels not being on the same filesystem,
@@ -194,7 +194,7 @@ class MappingKernelManager(MultiKernelManager):
         self._kernel_connections.pop(kernel_id, None)
         self._kernel_ports.pop(kernel_id, None)
 
-    async def start_kernel(self, kernel_id=None, path=None, **kwargs):
+    async def _async_start_kernel(self, kernel_id=None, path=None, **kwargs):
         """Start a kernel for a session and return its kernel_id.
 
         Parameters
@@ -212,10 +212,10 @@ class MappingKernelManager(MultiKernelManager):
         """
         if kernel_id is None or kernel_id not in self:
             if path is not None:
-                kwargs["cwd"] = self.cwd_for_path(path)
+                kwargs["cwd"] = self.cwd_for_path(path, env=kwargs.get("env", {}))
             if kernel_id is not None:
                 kwargs["kernel_id"] = kernel_id
-            kernel_id = await ensure_async(self.pinned_superclass.start_kernel(self, **kwargs))
+            kernel_id = await self.pinned_superclass._async_start_kernel(self, **kwargs)
             self._kernel_connections[kernel_id] = 0
             task = asyncio.create_task(self._finish_kernel_start(kernel_id))
             if not getattr(self, "use_pending_kernels", None):
@@ -224,18 +224,18 @@ class MappingKernelManager(MultiKernelManager):
                 self._pending_kernel_tasks[kernel_id] = task
             # add busy/activity markers:
             kernel = self.get_kernel(kernel_id)
-            kernel.execution_state = "starting"
-            kernel.reason = ""
-            kernel.last_activity = utcnow()
-            self.log.info("Kernel started: %s" % kernel_id)
-            self.log.debug("Kernel args: %r" % kwargs)
+            kernel.execution_state = "starting"  # type:ignore[attr-defined]
+            kernel.reason = ""  # type:ignore[attr-defined]
+            kernel.last_activity = utcnow()  # type:ignore[attr-defined]
+            self.log.info("Kernel started: %s", kernel_id)
+            self.log.debug("Kernel args: %r", kwargs)
 
             # Increase the metric of number of kernels running
             # for the relevant kernel type by 1
             KERNEL_CURRENTLY_RUNNING_TOTAL.labels(type=self._kernels[kernel_id].kernel_name).inc()
 
         else:
-            self.log.info("Using existing kernel: %s" % kernel_id)
+            self.log.info("Using existing kernel: %s", kernel_id)
 
         # Initialize culling if not already
         if not self._initialized_culler:
@@ -243,13 +243,18 @@ class MappingKernelManager(MultiKernelManager):
 
         return kernel_id
 
+    start_kernel = _async_start_kernel
+
     async def _finish_kernel_start(self, kernel_id):
         km = self.get_kernel(kernel_id)
         if hasattr(km, "ready"):
+            ready = km.ready
+            if not isinstance(ready, asyncio.Future):
+                ready = asyncio.wrap_future(ready)
             try:
-                await km.ready
-            except Exception as e:
-                self.log.exception(e)
+                await ready
+            except Exception:
+                self.log.exception("Error waiting for kernel manager ready")
                 return
 
         self._kernel_ports[kernel_id] = km.ports
@@ -276,7 +281,7 @@ class MappingKernelManager(MultiKernelManager):
         changed_ports = self._get_changed_ports(kernel_id)
         if changed_ports:
             # If changed, update captured ports and return True, else return False.
-            self.log.debug(f"Port change detected for kernel: {kernel_id}")
+            self.log.debug("Port change detected for kernel: %s", kernel_id)
             self._kernel_ports[kernel_id] = changed_ports
             return True
         return False
@@ -387,7 +392,7 @@ class MappingKernelManager(MultiKernelManager):
                 buffer_info["session_key"],
             )
 
-    def shutdown_kernel(self, kernel_id, now=False, restart=False):
+    async def _async_shutdown_kernel(self, kernel_id, now=False, restart=False):
         """Shutdown a kernel by kernel_id"""
         self._check_kernel_id(kernel_id)
 
@@ -398,16 +403,20 @@ class MappingKernelManager(MultiKernelManager):
         if kernel_id in self._pending_kernel_tasks:
             task = self._pending_kernel_tasks.pop(kernel_id)
             task.cancel()
-        else:
-            self.stop_watching_activity(kernel_id)
-            self.stop_buffering(kernel_id)
 
-        self.pinned_superclass.shutdown_kernel(self, kernel_id, now=now, restart=restart)
+        self.stop_watching_activity(kernel_id)
+        self.stop_buffering(kernel_id)
 
-    async def restart_kernel(self, kernel_id, now=False):
+        return await self.pinned_superclass._async_shutdown_kernel(
+            self, kernel_id, now=now, restart=restart
+        )
+
+    shutdown_kernel = _async_shutdown_kernel
+
+    async def _async_restart_kernel(self, kernel_id, now=False):
         """Restart a kernel by kernel_id"""
         self._check_kernel_id(kernel_id)
-        await ensure_async(self.pinned_superclass.restart_kernel(self, kernel_id, now=now))
+        await self.pinned_superclass._async_restart_kernel(self, kernel_id, now=now)
         kernel = self.get_kernel(kernel_id)
         # return a Future that will resolve when the kernel has successfully restarted
         channel = kernel.connect_shell()
@@ -415,10 +424,11 @@ class MappingKernelManager(MultiKernelManager):
 
         def finish():
             """Common cleanup when restart finishes/fails for any reason."""
-            if not channel.closed():
+            if not channel.closed():  # type:ignore[operator]
                 channel.close()
             loop.remove_timeout(timeout)
             kernel.remove_restart_callback(on_restart_failed, "dead")
+            kernel._pending_restart_cleanup = None  # type:ignore[attr-defined]
 
         def on_reply(msg):
             self.log.debug("Kernel info reply received: %s", kernel_id)
@@ -439,8 +449,9 @@ class MappingKernelManager(MultiKernelManager):
                 future.set_exception(RuntimeError("Restart failed"))
 
         kernel.add_restart_callback(on_restart_failed, "dead")
+        kernel._pending_restart_cleanup = finish  # type:ignore[attr-defined]
         kernel.session.send(channel, "kernel_info_request")
-        channel.on_recv(on_reply)
+        channel.on_recv(on_reply)  # type:ignore[operator]
         loop = IOLoop.current()
         timeout = loop.add_timeout(loop.time() + self.kernel_info_timeout, on_timeout)
         # Re-establish activity watching if ports have changed...
@@ -449,6 +460,8 @@ class MappingKernelManager(MultiKernelManager):
             self.start_watching_activity(kernel_id)
         return future
 
+    restart_kernel = _async_restart_kernel
+
     def notify_connect(self, kernel_id):
         """Notice a new connection to a kernel"""
         if kernel_id in self._kernel_connections:
@@ -544,6 +557,8 @@ class MappingKernelManager(MultiKernelManager):
             if not kernel._activity_stream.socket.closed:
                 kernel._activity_stream.close()
             kernel._activity_stream = None
+        if getattr(kernel, "_pending_restart_cleanup", None):
+            kernel._pending_restart_cleanup()
 
     def initialize_culler(self):
         """Start idle culler if 'cull_idle_timeout' is greater than zero.
@@ -638,33 +653,37 @@ class MappingKernelManager(MultiKernelManager):
 
 # AsyncMappingKernelManager inherits as much as possible from MappingKernelManager,
 # overriding only what is different.
-class AsyncMappingKernelManager(MappingKernelManager, AsyncMultiKernelManager):
+class AsyncMappingKernelManager(MappingKernelManager, AsyncMultiKernelManager):  # type:ignore[misc]
     @default("kernel_manager_class")
     def _default_kernel_manager_class(self):
-        return "jupyter_client.ioloop.AsyncIOLoopKernelManager"
+        return "jupyter_server.services.kernels.kernelmanager.ServerKernelManager"
+
+    @validate("kernel_manager_class")
+    def _validate_kernel_manager_class(self, proposal):
+        km_class_value = proposal.value
+        km_class = import_item(km_class_value)
+        if not issubclass(km_class, ServerKernelManager):
+            warnings.warn(
+                f"KernelManager class '{km_class}' is not a subclass of 'ServerKernelManager'.  Custom "
+                "KernelManager classes should derive from 'ServerKernelManager' beginning with jupyter-server 2.0 "
+                "or risk missing functionality.  Continuing...",
+                FutureWarning,
+                stacklevel=3,
+            )
+        return km_class_value
 
     def __init__(self, **kwargs):
-        self.pinned_superclass = AsyncMultiKernelManager
+        self.pinned_superclass = MultiKernelManager
+        self._pending_kernel_tasks = {}
         self.pinned_superclass.__init__(self, **kwargs)
         self.last_kernel_activity = utcnow()
-        self._pending_kernel_tasks = {}
 
-    async def shutdown_kernel(self, kernel_id, now=False, restart=False):
-        """Shutdown a kernel by kernel_id"""
-        self._check_kernel_id(kernel_id)
 
-        # Decrease the metric of number of kernels
-        # running for the relevant kernel type by 1
-        KERNEL_CURRENTLY_RUNNING_TOTAL.labels(type=self._kernels[kernel_id].kernel_name).dec()
+class ServerKernelManager(AsyncIOLoopKernelManager):
 
-        if kernel_id in self._pending_kernel_tasks:
-            task = self._pending_kernel_tasks.pop(kernel_id)
-            task.cancel()
-        else:
-            self.stop_watching_activity(kernel_id)
-            self.stop_buffering(kernel_id)
-
-        # Finish shutting down the kernel before clearing state to avoid a race condition.
-        return await self.pinned_superclass.shutdown_kernel(
-            self, kernel_id, now=now, restart=restart
-        )
+    # Define activity-related attributes:
+    execution_state = Unicode(
+        None, allow_none=True, help="The current execution state of the kernel"
+    )
+    reason = Unicode("", help="The reason for the last failure against the kernel")
+    last_activity = Instance(datetime, help="The last activity on the kernel")
