@@ -1,6 +1,7 @@
 """A base class session manager."""
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+import os
 import pathlib
 import uuid
 
@@ -13,12 +14,12 @@ except ImportError:
 from dataclasses import dataclass, fields
 from typing import Union
 
+from jupyter_core.utils import ensure_async
 from tornado import web
 from traitlets import Instance, TraitError, Unicode, validate
 from traitlets.config.configurable import LoggingConfigurable
 
 from jupyter_server.traittypes import InstanceFromClasses
-from jupyter_server.utils import ensure_async
 
 
 class KernelSessionRecordConflict(Exception):
@@ -243,7 +244,7 @@ class SessionManager(LoggingConfigurable):
         return exists
 
     def new_session_id(self):
-        "Create a uuid for a new session"
+        """Create a uuid for a new session"""
         return str(uuid.uuid4())
 
     async def create_session(
@@ -267,12 +268,19 @@ class SessionManager(LoggingConfigurable):
         self._pending_sessions.remove(record)
         return result
 
+    def get_kernel_env(self, path):
+        """Return the environment variables that need to be set in the kernel"""
+        return {**os.environ, "JPY_SESSION_NAME": path}
+
     async def start_kernel_for_session(self, session_id, path, name, type, kernel_name):
         """Start a new kernel for a given session."""
         # allow contents manager to specify kernels cwd
-        kernel_path = self.contents_manager.get_kernel_path(path=path)
+        kernel_path = await ensure_async(self.contents_manager.get_kernel_path(path=path))
+        kernel_env = self.get_kernel_env(path)
         kernel_id = await self.kernel_manager.start_kernel(
-            path=kernel_path, kernel_name=kernel_name
+            path=kernel_path,
+            kernel_name=kernel_name,
+            env=kernel_env,
         )
         return kernel_id
 
@@ -354,7 +362,7 @@ class SessionManager(LoggingConfigurable):
         try:
             model = await self.row_to_model(row)
         except KeyError as e:
-            raise web.HTTPError(404, "Session not found: %s" % str(e))
+            raise web.HTTPError(404, "Session not found: %s" % str(e)) from e
         return model
 
     async def update_session(self, session_id, **kwargs):
