@@ -1,28 +1,28 @@
 """Notebook related utilities"""
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-import asyncio
 import errno
 import importlib.util
-import inspect
 import os
 import socket
 import sys
-from _frozen_importlib_external import _NamespacePath
+import warnings
 from contextlib import contextmanager
-from urllib.parse import quote
-from urllib.parse import SplitResult
-from urllib.parse import unquote
-from urllib.parse import urljoin  # noqa: F401
-from urllib.parse import urlparse
-from urllib.parse import urlsplit
-from urllib.parse import urlunsplit
-from urllib.request import pathname2url  # noqa: F401
+from urllib.parse import (
+    SplitResult,
+    quote,
+    unquote,
+    urljoin,  # noqa
+    urlparse,
+    urlsplit,
+    urlunsplit,
+)
+from urllib.request import pathname2url  # noqa
 
+from _frozen_importlib_external import _NamespacePath
+from jupyter_core.utils import ensure_async
 from packaging.version import Version
-from tornado.httpclient import AsyncHTTPClient
-from tornado.httpclient import HTTPClient
-from tornado.httpclient import HTTPRequest
+from tornado.httpclient import AsyncHTTPClient, HTTPClient, HTTPRequest
 from tornado.netutil import Resolver
 
 
@@ -118,7 +118,7 @@ def to_os_path(path, root=""):
     parts = path.strip("/").split("/")
     parts = [p for p in parts if p != ""]  # remove duplicate splits
     path = os.path.join(root, *parts)
-    return path
+    return os.path.normpath(path)
 
 
 def to_api_path(os_path, root=""):
@@ -156,7 +156,7 @@ def _check_pid_win32(pid):
 
     # OpenProcess returns 0 if no such process (of ours) exists
     # positive int otherwise
-    return bool(ctypes.windll.kernel32.OpenProcess(1, 0, pid))
+    return bool(ctypes.windll.kernel32.OpenProcess(1, 0, pid))  # type:ignore[attr-defined]
 
 
 def _check_pid_posix(pid):
@@ -180,90 +180,14 @@ else:
     check_pid = _check_pid_posix
 
 
-async def ensure_async(obj):
-    """Convert a non-awaitable object to a coroutine if needed,
-    and await it if it was not already awaited.
-    """
-    if inspect.isawaitable(obj):
-        try:
-            result = await obj
-        except RuntimeError as e:
-            if str(e) == "cannot reuse already awaited coroutine":
-                # obj is already the coroutine's result
-                return obj
-            raise
-        return result
-    # obj doesn't need to be awaited
-    return obj
-
-
-def run_sync(maybe_async):
-    """If async, runs maybe_async and blocks until it has executed,
-    possibly creating an event loop.
-    If not async, just returns maybe_async as it is the result of something
-    that has already executed.
-
-    Parameters
-    ----------
-    maybe_async : async or non-async object
-        The object to be executed, if it is async.
-
-    Returns
-    -------
-    result
-        Whatever the async object returns, or the object itself.
-    """
-    if not inspect.isawaitable(maybe_async):
-        # that was not something async, just return it
-        return maybe_async
-    # it is async, we need to run it in an event loop
-
-    def wrapped():
-        create_new_event_loop = False
-        try:
-            loop = asyncio.get_event_loop()
-        except RuntimeError:
-            create_new_event_loop = True
-        else:
-            if loop.is_closed():
-                create_new_event_loop = True
-        if create_new_event_loop:
-            loop = asyncio.new_event_loop()
-            asyncio.set_event_loop(loop)
-        try:
-            result = loop.run_until_complete(maybe_async)
-        except RuntimeError as e:
-            if str(e) == "This event loop is already running":
-                # just return a Future, hoping that it will be awaited
-                result = asyncio.ensure_future(maybe_async)
-            else:
-                raise e
-        return result
-
-    return wrapped()
-
-
 async def run_sync_in_loop(maybe_async):
-    """Runs a function synchronously whether it is an async function or not.
-
-    If async, runs maybe_async and blocks until it has executed.
-
-    If not async, just returns maybe_async as it is the result of something
-    that has already executed.
-
-    Parameters
-    ----------
-    maybe_async : async or non-async object
-        The object to be executed, if it is async.
-
-    Returns
-    -------
-    result
-        Whatever the async object returns, or the object itself.
-    """
-    if not inspect.isawaitable(maybe_async):
-        return maybe_async
-    return await maybe_async
+    """**DEPRECATED**: Use ``ensure_async`` from jupyter_core instead."""
+    warnings.warn(
+        "run_sync_in_loop is deprecated since Jupyter Server 2.0, use 'ensure_async' from jupyter_core instead",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return ensure_async(maybe_async)
 
 
 def urlencode_unix_socket_path(socket_path):
@@ -289,7 +213,7 @@ def unix_socket_in_use(socket_path):
     try:
         sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         sock.connect(socket_path)
-    except socket.error:
+    except OSError:
         return False
     else:
         return True
@@ -344,7 +268,7 @@ def _request_for_tornado_client(urlstring, method="GET", body=None, headers=None
 
     # Yield the request for the given client.
     url = urlunsplit(parts)
-    request = HTTPRequest(url, method=method, body=body, headers=headers)
+    request = HTTPRequest(url, method=method, body=body, headers=headers, validate_cert=False)
     yield request
 
 
@@ -353,7 +277,9 @@ def fetch(urlstring, method="GET", body=None, headers=None):
     Send a HTTP, HTTPS, or HTTP+UNIX request
     to a Tornado Web Server. Returns a tornado HTTPResponse.
     """
-    with _request_for_tornado_client(urlstring) as request:
+    with _request_for_tornado_client(
+        urlstring, method=method, body=body, headers=headers
+    ) as request:
         response = HTTPClient(AsyncHTTPClient).fetch(request)
     return response
 
@@ -363,7 +289,9 @@ async def async_fetch(urlstring, method="GET", body=None, headers=None, io_loop=
     Send an asynchronous HTTP, HTTPS, or HTTP+UNIX request
     to a Tornado Web Server. Returns a tornado HTTPResponse.
     """
-    with _request_for_tornado_client(urlstring) as request:
+    with _request_for_tornado_client(
+        urlstring, method=method, body=body, headers=headers
+    ) as request:
         response = await AsyncHTTPClient(io_loop).fetch(request)
     return response
 
@@ -432,7 +360,7 @@ def filefind(filename, path_dirs=None):
         if os.path.isfile(testname):
             return os.path.abspath(testname)
 
-    raise IOError("File %r does not exist in any of the search paths: %r" % (filename, path_dirs))
+    raise OSError(f"File {filename!r} does not exist in any of the search paths: {path_dirs!r}")
 
 
 def expand_path(s):
@@ -476,8 +404,8 @@ def import_item(name):
         module = __import__(package, fromlist=[obj])
         try:
             pak = getattr(module, obj)
-        except AttributeError:
-            raise ImportError("No module named %s" % obj)
+        except AttributeError as e:
+            raise ImportError("No module named %s" % obj) from e
         return pak
     else:
         # called with un-dotted string
