@@ -1,16 +1,13 @@
-# encoding: utf-8
 """Tornado handlers for viewing HTML files."""
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+from jupyter_core.utils import ensure_async
 from tornado import web
 
-from ..base.handlers import JupyterHandler
-from ..base.handlers import path_regex
-from ..utils import ensure_async
-from ..utils import url_escape
-from ..utils import url_path_join
 from jupyter_server.auth import authorized
 
+from ..base.handlers import JupyterHandler, path_regex
+from ..utils import url_escape, url_path_join
 
 AUTH_RESOURCE = "contents"
 
